# -*- coding: utf8 -*-
# Copyright (c) 2018-2020 THL A29 Limited, a Tencent company. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#    http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

class RunUserEntrypointError(Exception):
    def __init__(self, command, output):
        self.command = command
        self.output = output
        super(RunUserEntrypointError, self).__init__()

    def __str__(self):
        error_msg = "\n%s" % self.output if self.output else ""
        message = '%s:\nCommand "%s"%s' % (type(self).__name__, self.command, error_msg)
        return message.strip()
